@@ -1,3 +1,21 @@
+/*
+ * Copyright 2010 Felipe Crochik <foss@crochik.com>
+ * All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "mainwindow.h"
 
 #ifdef Q_WS_MAEMO_5
@@ -10,6 +28,8 @@
 #endif
 
 #include <QDebug>
+#include <QApplication>
+#include <QDesktopWidget>
 
 #include "fullscreenexitbutton.h"
 
@@ -24,19 +44,22 @@ MainWindow::MainWindow(MainWindow *pParent) :
     QMainWindow(),
     m_webPage(this)
 {
+    m_pBookmarks = NULL;
     m_pGraphicsView = NULL;
     m_pGWebView = NULL;
     m_pWebView = NULL;
+
+#ifdef Q_WS_MAEMO_5
+    m_pOrientationSelector = NULL;
+    m_pUserAgentSelector = NULL;
+#endif
+
+    m_pParent = pParent;
+
     m_isUsingGraphicsView = false;
 
     setWindowTitle("Macuco");
-    m_pParent = pParent;
 
-    createMenu();
-
-    // loading label...
-    m_pLoadingLabel = new QLabel(this);
-    m_pLoadingLabel->setPixmap( QPixmap(QString::fromUtf8(":/images/spinner.png")) );
 
 #ifdef Q_WS_MAEMO_5
     FullScreenExitButton *pButt = new FullScreenExitButton(this);
@@ -45,30 +68,39 @@ MainWindow::MainWindow(MainWindow *pParent) :
     grabZoomKeys(true);
 
     setAttribute(Qt::WA_Maemo5StackedWindow);
-
-#else // !Q_WS_MAEMO_5
-    // error box
-    m_pErrorBox = new ErrorBox(this);
-    m_pErrorBox->hide();
-    connect(m_pErrorBox->getRetryButton(), SIGNAL(clicked()), this, SLOT(reload()));
-
-    // tool bar
-    m_pToolbar = new Toolbar(this);
-    m_pToolbar->hide();
-
-    m_pMiniToolbar = new MinifiedToolbar(this);
-    m_pMiniToolbar->show();
-
-    connect(m_pMiniToolbar->getShowToolbarButton(), SIGNAL(clicked()), this, SLOT(showToolbar()));
-    connect(m_pToolbar->getHideToolbarButton(), SIGNAL(clicked()), this, SLOT(hideToolbar()));
-
-    connect(m_pToolbar->getBackButton(), SIGNAL(clicked()), m_pView, SLOT(back()));
-    connect(m_pToolbar->getFowardButton(), SIGNAL(clicked()), m_pView, SLOT(forward()));
-    connect(m_pToolbar->getReloadButton(), SIGNAL(clicked()), m_pView, SLOT(reload()));
-
-    connect(m_pToolbar, SIGNAL(urlChanged(QString)), this, SLOT(visitUrl(QString)));
 #endif
 
+#ifdef Q_WS_MAEMO_5
+    QString storagePath = "/home/user/.macuco";
+#else
+    QString storagePath = QDesktopServices::storageLocation(QDesktopServices::CacheLocation) +
+            QDir::separator() + "macuco";
+            //QApplication::applicationDirPath() + "/cache";
+#endif
+
+    qDebug() << "cache location: " << storagePath;
+    QWebSettings::enablePersistentStorage(storagePath);
+
+    // disk cache
+    // QString storagePath = QWebSettings::offlineWebApplicationCachePath();
+    qDebug() << "disk cache: " << storagePath;
+    QNetworkDiskCache *pCache = new QNetworkDiskCache(this);
+    pCache->setCacheDirectory(storagePath);
+    m_webPage.networkAccessManager()->setCache(pCache);
+
+    // m_webPage.setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
+    // m_webPage.settings()->setAttribute(QWebSettings::PluginsEnabled, true);
+    // m_webPage.setForwardUnsupportedContent(true);
+    m_webPage.settings()->setAttribute(QWebSettings::DeveloperExtrasEnabled, true);
+
+    connect(m_webPage.networkAccessManager(), SIGNAL(finished(QNetworkReply*)),
+        this, SLOT(finishedRequest(QNetworkReply*)));
+
+    // connect(&m_webPage, SIGNAL(linkClicked(QUrl)), this, SLOT(linkClicked(QUrl)));
+    connect(&m_webPage, SIGNAL(unsupportedContent(QNetworkReply*)), this, SLOT(unsupportedContent(QNetworkReply*)));
+    connect(&m_webPage, SIGNAL(downloadRequested(QNetworkRequest)), this, SLOT(downloadRequested(QNetworkRequest)));
+
+
     if ( m_pParent ) {
         // copy settings from mainwindow
         m_zoomFactor = m_pParent->m_zoomFactor;
@@ -83,39 +115,33 @@ MainWindow::MainWindow(MainWindow *pParent) :
 #endif
 
     } else {
-        m_zoomFactor = 2;
+	m_zoomFactor = 1;
         m_pBookmarks = new QList<Bookmark>();
 
         // load settings
         loadSettings();
     }
 
-    connect(m_webPage.networkAccessManager(), SIGNAL(finished(QNetworkReply*)),
-        this, SLOT(finishedRequest(QNetworkReply*)));
-
-    // m_webPage.setLinkDelegationPolicy(QWebPage::DelegateAllLinks);
-    // m_webPage.settings()->setAttribute(QWebSettings::PluginsEnabled, true);
-    // m_webPage.setForwardUnsupportedContent(true);
-    m_webPage.settings()->setAttribute(QWebSettings::DeveloperExtrasEnabled, true);
-
-    // offline?
-    // QWebSettings::OfflineStorageDatabaseEnabled
-    // QWebSettings::OfflineWebApplicationCacheEnabled
-    // QWebSettings::enablePersistentStorage(path);
-
-    // connect(&m_webPage, SIGNAL(linkClicked(QUrl)), this, SLOT(linkClicked(QUrl)));
-    connect(&m_webPage, SIGNAL(unsupportedContent(QNetworkReply*)), this, SLOT(unsupportedContent(QNetworkReply*)));
-    connect(&m_webPage, SIGNAL(downloadRequested(QNetworkRequest)), this, SLOT(downloadRequested(QNetworkRequest)));
-
     if ( !m_pParent ) {
         setWidget();
-        visitUrl("http://www.crochik.com/macuco");
+        // visitUrl("http://www.crochik.com/macuco");
+        on_actionHome_triggered();
 
     } else {
 
         m_isUsingGraphicsView = m_pParent->m_isUsingGraphicsView;
         setWidget();
     }
+
+#ifdef Q_WS_X11
+#ifndef Q_WS_MAEMO_5
+    // meego?
+    setGeometry(QRect(QPoint(0,0),QApplication::desktop()->size()));
+#endif
+#endif
+
+    createMenu();
+    updateTitle();
 }
 
 MainWindow::~MainWindow() {
@@ -135,10 +161,18 @@ MainWindow::~MainWindow() {
 }
 
 void MainWindow::createMenu() {
-    menuBar()->addAction("Home", this, SLOT(on_actionHome_triggered()));
-    menuBar()->addAction("Go To...", this, SLOT(on_actionGo_To_triggered()));
-    menuBar()->addAction("Bookmarks", this, SLOT(on_actionBookmarks_triggered()));
-    menuBar()->addAction("Add Bookmark", this, SLOT(on_actionAdd_Bookmark_triggered()));
+    QMenu *pMenu = menuBar()->addMenu("&File");
+    pMenu->addAction("Home", this, SLOT(on_actionHome_triggered()));
+    pMenu->addAction("Go To...", this, SLOT(on_actionGo_To_triggered()));
+
+#ifndef Q_WS_MAEMO_5
+    pMenu->addSeparator();
+    pMenu->addAction("&Exit", this, SLOT(close()));
+#endif
+
+    pMenu = menuBar()->addMenu("&Bookmarks");
+    pMenu->addAction("Bookmarks", this, SLOT(on_actionBookmarks_triggered()));
+    pMenu->addAction("Add Bookmark", this, SLOT(on_actionAdd_Bookmark_triggered()));
 
 #ifdef Q_WS_MAEMO_5
     menuBar()->addAction("Fullscreen", this, SLOT(fullscreen_triggered()));
@@ -151,22 +185,36 @@ void MainWindow::createMenu() {
 
     m_pUserAgentSelector = new PickSelectorAction(this);
     m_pUserAgentSelector->setText("User Agent");
-    m_pUserAgentSelector->items() << "IPhone" << "Default";
+    m_pUserAgentSelector->items() << "iPhone" << "Default";
     connect(m_pUserAgentSelector, SIGNAL(selectionChanged(int)), this, SLOT(userAgent_selectionChanged(int)));
     menuBar()->addAction(m_pUserAgentSelector);
 #else
-    QAction *pFullscreen = menuBar()->addAction("Fullscreen");
+    pMenu = menuBar()->addMenu("&View");
+    QAction *pFullscreen = pMenu->addAction("Fullscreen");
     pFullscreen->setCheckable(true);
     connect(pFullscreen, SIGNAL(toggled(bool)), this, SLOT(setFullScreen(bool)));
 
-    // user agent
-    // ...
+    pMenu->addSeparator();
+
+    QActionGroup *pActionGroup = new QActionGroup(pMenu);
+    QAction *pAct1 = pActionGroup->addAction("iPhone User Agent");
+    QAction *pAct2 = pActionGroup->addAction("Default User Agent");
+    connect(pActionGroup, SIGNAL(selected(QAction*)), SLOT(onUserAgentChanged(QAction*)));
+    pMenu->addActions(pActionGroup->actions());
+    pAct1->setCheckable(true);
+    pAct2->setCheckable(true);
+    if ( m_webPage.isIphoneUserAgent()) pAct1->setChecked(true); else pAct2->setChecked(true);
+
+    #ifdef QT_DEBUG
+        pMenu->addSeparator();
+        pMenu->addAction("Switch View", this, SLOT(switchView()));
+    #endif
 
 #endif
 
 #ifdef QT_DEBUG
-    menuBar()->addAction("Switch View", this, SLOT(switchView()));
-    menuBar()->addAction("About Qt..", this, SLOT(aboutQt()));
+    pMenu = menuBar()->addMenu("&About");
+    pMenu->addAction("About Qt..", this, SLOT(aboutQt()));
 #endif
 }
 
@@ -230,6 +278,13 @@ void MainWindow::aboutQt() {
     QMessageBox::aboutQt(this, "Qt");
 }
 
+void MainWindow::onUserAgentChanged(QAction *pAction) {
+    qDebug() << pAction->text();
+
+    m_webPage.setIPhoneUserAgent(pAction->text().startsWith("iPhone"));
+    updateTitle();
+}
+
 void MainWindow::userAgent_selectionChanged(int index) {
     switch ( index ) {
         case 0: // iphone
@@ -298,9 +353,29 @@ void MainWindow::loadSettings() {
     orientation_selectionChanged(orientation);
 #endif
 
+    qDebug() << "try to get icons";
+    // get page icons
+    // (work around for not getting the icons the first time - have to investigate)
+    for ( int c=0; c<m_pBookmarks->size(); c++ ) {
+        QWebSettings::iconForUrl((*m_pBookmarks)[c].url());
+    }
+
     qDebug() << ">> done";
 }
 
+void MainWindow::updateTitle() {
+#ifdef Q_WS_MAEMO_5
+
+#else
+    QString title = m_currentUrl.isEmpty() ? "Macuco" : (m_isUsingGraphicsView?m_pGWebView->title():m_pWebView->title());
+    if ( m_webPage.isIphoneUserAgent() ) title += " - iPhone";
+
+    title += QString(" - %1%").arg(QString::number(m_zoomFactor*100, 'f', 0));
+    setWindowTitle(title);
+#endif
+
+}
+
 void MainWindow::saveSettings() {
     QSettings settings("Macuco", "Macuco");
     settings.setValue("version", SETTINGS_VERSION);
@@ -322,12 +397,14 @@ void MainWindow::showLoading() {
 #ifdef Q_WS_MAEMO_5
     // http://qt.nokia.com/doc/qt-maemo-4.6/maemo5-windowstates.html
     setAttribute(Qt::WA_Maemo5ShowProgressIndicator, true);
-    if ( isFullScreen() ) m_pLoadingLabel->show();
 #else
+    setCursor(QCursor(Qt::WaitCursor));
+    /*
     m_pErrorBox->hide();
     hideOverlayOnPage();
 
     m_pLoadingLabel->show();
+    */
 #endif
 }
 
@@ -339,12 +416,15 @@ void MainWindow::changeUrl(QUrl url) {
 
     m_currentUrl = url;
 
+    /*
 #ifndef Q_WS_MAEMO_5
     m_pToolbar->getInputAddress()->setText( url.toString() );
 #endif
+    */
 }
 
 void MainWindow::visitUrl(QString url) {
+
     if ( url.isNull() || url.isEmpty() ) return;
 
     if ( m_isUsingGraphicsView )
@@ -358,24 +438,16 @@ void MainWindow::reload() {
         m_pGWebView->load(m_currentUrl);
     else
         m_pWebView->load(m_currentUrl);
-
+    /*
 #ifndef Q_WS_MAEMO_5
     m_pErrorBox->hide();
     hideOverlayOnPage();
-#endif
+#end
+    */
 }
 
 void MainWindow::adjustSizes() {
-    m_pLoadingLabel->setGeometry( width()/2 - 45, height()/2 - 45, 90, 90); //put the label on the center
-
-#ifndef Q_WS_MAEMO_5
-    m_pErrorBox->setGeometry( width()/2 - 150, height()/2 - 75, 300, 150); //put the error box on the center
-    m_pToolbar->setGeometry( 0, height() - 38, width(), 38 );
-    m_pMiniToolbar->setGeometry( 0, height() - 38, 30, 38 );
-#endif
-
-    if ( m_webPage.isIphoneUserAgent() ) zoomFit();
-
+    if ( !m_currentUrl.isEmpty() && m_webPage.isIphoneUserAgent() ) zoomFit();
 }
 
 void MainWindow::zoomFit() {
@@ -437,6 +509,8 @@ void MainWindow::setZoom(qreal zoomFactor) {
     m_zoomFactor = zoomFactor;
     if ( m_zoomFactor>.90 && m_zoomFactor<1.1) m_zoomFactor =1;
 
+    updateTitle();
+
     if ( m_isUsingGraphicsView ) {
         if ( !m_pGWebView ) return;
         m_pGWebView->setZoomFactor(m_zoomFactor);
@@ -609,7 +683,7 @@ void MainWindow::setAutoRotate(bool autoRotate /*=true*/) {
     if ( autoRotate ) {
         qDebug() << "set autorotate: on";
         setAttribute(Qt::WA_Maemo5AutoOrientation, true);
-        m_pOrientationSelector->setSelectedIndex(0);
+        if ( m_pOrientationSelector ) m_pOrientationSelector->setSelectedIndex(0);
         return;
     }
 
@@ -651,42 +725,6 @@ void MainWindow::grabZoomKeys(bool grab)
             1);
 
 }
-
-#else  // !Q_WS_MAEMO_5
-void MainWindow::showOverlayOnPage()
-{
-    //TODO: is there a better way to put a transparent overlay on the screen?
-    QString script =
-           "newDiv = document.createElement(\"div\");"
-           "newDiv = document.createElement(\"div\");"
-           "newDiv.innerHTML = \"<div id='divOverlay' style='width: 100%; height: 100%; background: white;"
-           "position: absolute; top: 0; left: 0; opacity: 0.8'></div>\";"
-           "my_div = document.getElementById(\"body\");"
-           "document.body.appendChild(newDiv);";
-
-    m_webPage.mainFrame()->evaluateJavaScript(script);
-}
-
-void MainWindow::hideOverlayOnPage()
-{
-    QString script = "node = document.getElementById(\"divOverlay\");"
-                     "node.parentNode.removeChild(node);";
-
-    m_webPage.mainFrame()->evaluateJavaScript(script);
-}
-
-void MainWindow::showToolbar()
-{
-    m_pToolbar->show();
-    m_pMiniToolbar->hide();
-}
-
-void MainWindow::hideToolbar()
-{
-    m_pToolbar->hide();
-    m_pMiniToolbar->show();
-}
-
 #endif
 
 
@@ -698,7 +736,7 @@ void MainWindow::setUserAgent(bool iphone/*=true*/) {
     m_webPage.setIPhoneUserAgent(iphone);
 
 #ifdef Q_WS_MAEMO_5
-    m_pUserAgentSelector->setSelectedIndex(iphone?0:1);
+    if ( m_pUserAgentSelector ) m_pUserAgentSelector->setSelectedIndex(iphone?0:1);
 #endif
 }
 
@@ -732,8 +770,6 @@ void MainWindow::finishedRequest(QNetworkReply* reply)
 }
 
 void MainWindow::hideLoading(bool success) {
-    m_pLoadingLabel->hide();
-
     setZoom(m_zoomFactor);
 
 #ifdef Q_WS_MAEMO_5
@@ -751,16 +787,10 @@ void MainWindow::hideLoading(bool success) {
         );
         return;
     }
-
 #else
-    hideOverlayOnPage();
+    setCursor(QCursor(Qt::ArrowCursor));
 
-    if (success) {
-        m_pErrorBox->hide();
-    } else {
-        showOverlayOnPage();
-        m_pErrorBox->show();
-    }
+    if ( !success ) QMessageBox::warning(this, "Loading Error", m_errors.join("\n"));
 #endif
 }
 
